@@ -39,7 +39,6 @@ from collections.abc import Callable
 from functools import wraps
 from typing import IO, TYPE_CHECKING, Any, TypeVar, cast
 
-from asgiref.sync import sync_to_async
 from azure.identity import ClientSecretCredential, DefaultAzureCredential
 from azure.identity.aio import (
     ClientSecretCredential as AsyncClientSecretCredential,
@@ -48,6 +47,7 @@ from azure.identity.aio import (
 from azure.mgmt.datafactory import DataFactoryManagementClient
 from azure.mgmt.datafactory.aio import DataFactoryManagementClient as AsyncDataFactoryManagementClient
 
+from airflow.providers.common.compat.connection import get_async_connection
 from airflow.providers.common.compat.sdk import AirflowException, BaseHook
 from airflow.providers.microsoft.azure.utils import (
     add_managed_identity_connection_widgets,
@@ -1089,7 +1089,7 @@ def provide_targeted_factory_async(func: T) -> T:
             # Check if arg was not included in the function signature or, if it is, the value is not provided.
             if arg not in bound_args.arguments or bound_args.arguments[arg] is None:
                 self = args[0]
-                conn = await sync_to_async(self.get_connection)(self.conn_id)
+                conn = await get_async_connection(self.conn_id)
                 extras = conn.extra_dejson
                 default_value = extras.get(default_key) or extras.get(
                     f"extra__azure_data_factory__{default_key}"
@@ -1121,12 +1121,26 @@ class AzureDataFactoryAsyncHook(AzureDataFactoryHook):
         self.conn_id = azure_data_factory_conn_id
         super().__init__(azure_data_factory_conn_id=azure_data_factory_conn_id)
 
+    async def __aenter__(self):
+        """Enter async context manager - returns self for use in async with blocks."""
+        return self
+
+    async def __aexit__(self, exc_type, exc_val, exc_tb):
+        """Exit async context manager - closes the async connection."""
+        await self.close()
+
+    async def close(self) -> None:
+        """Close the async connection to Azure Data Factory."""
+        if self._async_conn is not None:
+            await self._async_conn.close()
+            self._async_conn = None
+
     async def get_async_conn(self) -> AsyncDataFactoryManagementClient:
         """Get async connection and connect to azure data factory."""
         if self._async_conn is not None:
             return self._async_conn
 
-        conn = await sync_to_async(self.get_connection)(self.conn_id)
+        conn = await get_async_connection(self.conn_id)
         extras = conn.extra_dejson
         tenant = get_field(extras, "tenantId")
 
