@@ -23,14 +23,13 @@ from collections.abc import Sequence
 from functools import cached_property
 from typing import TYPE_CHECKING
 
-from airflow.configuration import conf
-from airflow.models import BaseOperator
+from airflow.providers.common.compat.sdk import BaseOperator, conf
 from airflow.providers.microsoft.azure.hooks.adx import AzureDataExplorerHook
 
 if TYPE_CHECKING:
     from azure.kusto.data._models import KustoResultTable
 
-    from airflow.utils.context import Context
+    from airflow.sdk import Context
 
 
 class AzureDataExplorerQueryOperator(BaseOperator):
@@ -81,5 +80,4 @@ class AzureDataExplorerQueryOperator(BaseOperator):
         # TODO: Remove this after minimum Airflow version is 3.0
         if conf.getboolean("core", "enable_xcom_pickling", fallback=False):
             return response.primary_results[0]
-        else:
-            return str(response.primary_results[0])
+        return str(response.primary_results[0])
